@@ -242,6 +242,8 @@ def addASG_DFARule( AssemblySystemName, component,rule,model,KBPath,API):
                         #check if rule selector is available
                         try:
                             cmp = ASG_temp.hasRuleSelector
+                            # --!HACK!--:check if an DFARule class already exists, if not, generate a placeholder
+                            model,cmp = DFA_Hack(API=API,AssemblySystemName=AssemblySystemName,classID="RuleSelector",cmp=cmp,model=model)
                             addDFArule(cmp,rule,API)
                             updateKBv6(KBPath,model)
                         except:
@@ -252,6 +254,8 @@ def addASG_DFARule( AssemblySystemName, component,rule,model,KBPath,API):
                         #check if rule selector is available
                         try:
                             cmp = ASG_temp.hasRuleEvaluator
+                            # --!HACK!--:check if an DFARule class already exists, if not, generate a placeholder
+                            model,cmp = DFA_Hack(API=API, AssemblySystemName=AssemblySystemName, classID="RuleEvaluator",cmp=cmp,model=model)
                             addDFArule(cmp, rule,API)
                             updateKBv6(KBPath,model)
                         except:
@@ -291,6 +295,8 @@ def addASG_DFARules( AssemblySystemName, component,ruleList,model,KBPath,API):
                         #check if rule selector is available
                         try:
                             cmp = ASG_temp.hasRuleSelector
+                            # --!HACK!--:check if an DFARule class already exists, if not, generate a placeholder
+                            model, cmp = DFA_Hack(API=API, AssemblySystemName=AssemblySystemName,classID="RuleSelector", cmp=cmp, model=model)
                             for rule in ruleList:
                                 addDFArule(cmp,rule,API)
                             updateKBv6(KBPath,model)
@@ -302,6 +308,8 @@ def addASG_DFARules( AssemblySystemName, component,ruleList,model,KBPath,API):
                         #check if rule selector is available
                         try:
                             cmp = ASG_temp.hasRuleEvaluator
+                            # --!HACK!--:check if an DFARule class already exists, if not, generate a placeholder
+                            model, cmp = DFA_Hack(API=API, AssemblySystemName=AssemblySystemName,classID="RuleEvaluator", cmp=cmp, model=model)
                             for rule in ruleList:
                                 addDFArule(cmp, rule, API)
                             updateKBv6(KBPath,model)
@@ -355,4 +363,19 @@ def addDFArule(ruleContainer, newRule,API):
         # update the options
         # TODO!!!
 
-    ruleContainer.hasDFARule.items.append(rule)
\ No newline at end of file
+    ruleContainer.hasDFARule.items.append(rule)
+
+def DFA_Hack(API,AssemblySystemName,classID,cmp,model):
+    """
+      For internal use only!
+    """
+    if len(cmp.hasDFARule.items) == 0:
+        print("DOES NOT EXIST, PERFORM HACK")
+        model = EmptySubClassHack(API=API, classID=classID, className=cmp.hasName)
+        for assemblySystem in model.includesAssemblySystem.items:
+            if AssemblySystemName == assemblySystem.hasName:
+                ASG_temp = assemblySystem.hasAssemblyAlgorithmConfiguration
+                cmp = ASG_temp.hasRuleSelector
+        return model,cmp
+    else:
+        return model,cmp
\ No newline at end of file
