@@ -25,12 +25,16 @@ def updateKBv6(KB_path,model):
     postProcess_DFARule(KB_path)
     postProcess_OptimizationProblem(KB_path)
     postProcess_AssemblySystem(KB_path)
+    postProcess_ASG(KB_path)
+    postProcess_StopCondition(KB_path)
     postProcess_platformRoot(KB_path)
     #--DEBUG EXPERIMENT:use a blank template of optimizationProblem <includesOptimizationProblem/> and delete it
     postProcess_EmptyOptimization(KB_path)
     postProcess_EmptyProduct(KB_path)
     postProcess_EmptyProductPart(KB_path)
     postProcess_EmptyASG(KB_path)
+    #postProcess_EmptyStopCriterion(KB_path)
+    postProcess_EmptyDFARule(KB_path)
     postProcess_Line(KB_path)
     #--END DEBUG EXPERIMENT
 
@@ -345,6 +349,26 @@ def postProcess_AssemblySystem( output_file):
 
     file.close()
 
+def postProcess_ASG( output_file):
+    """
+        Custom postprocessing function to clean the xmi file defined by an Ecore model
+
+        :param string path: path of the output xmi file
+    """
+    # open file to manually add missing attribute in root element (@ToDo clean this up)
+    with open(output_file, "r") as file:
+        data = file.read()
+
+    # Prefix replacement
+    target = '</paco:AssemblyAlgorithmConfiguration'
+    replacement = '</includesAssemblyAlgorithmConfiguration'
+
+    with open(output_file, "w") as file:
+        data_new = data.replace(target, replacement)
+        file.write(data_new)
+
+    file.close()
+
 #--DEBUG EXPERIMENT:use a blank template of optimizationProblem <includesOptimizationProblem/> and delete it
 def postProcess_EmptyOptimization( output_file):
     """
@@ -426,6 +450,46 @@ def postProcess_EmptyASG( output_file):
 
     file.close()
 
+def postProcess_EmptyStopCriterion( output_file):
+    """
+        Custom postprocessing function to clean the xmi file defined by an Ecore model
+
+        :param string path: path of the output xmi file
+    """
+    # open file to manually add missing attribute in root element (@ToDo clean this up)
+    with open(output_file, "r") as file:
+        data = file.read()
+
+    # Prefix replacement
+    target = '<hasStopCriterion/>'
+    replacement = ''
+
+    with open(output_file, "w") as file:
+        data_new = data.replace(target, replacement)
+        file.write(data_new)
+
+    file.close()
+
+def postProcess_EmptyDFARule( output_file):
+    """
+        Custom postprocessing function to clean the xmi file defined by an Ecore model
+
+        :param string path: path of the output xmi file
+    """
+    # open file to manually add missing attribute in root element (@ToDo clean this up)
+    with open(output_file, "r") as file:
+        data = file.read()
+
+    # Prefix replacement
+    target = '<hasDFARule/>'
+    replacement = ''
+
+    with open(output_file, "w") as file:
+        data_new = data.replace(target, replacement)
+        file.write(data_new)
+
+    file.close()
+
 def postProcess_Line( output_file):
     """
         Custom postprocessing function to clean the xmi file defined by an Ecore model
@@ -474,4 +538,66 @@ def EmptyClassHack(API=None,elementName="OptimizationProblem"):
     API.model = importInstanceModel(API.ECORE_path, API.KB_path, VERBOSE=False)
     return API.model
 
+#END DEBUG EXPERIMENT
+
+#--DEBUG EXPERIMENT: hacking the empty subclass e.g. to update stopconditions or DFA rules
+def preProcess_emptySubClass(KB_file,classID ="Terminator" ,className="Default"):
+    """
+        Custom preprocessing function to generate an empty class element, used in the EmptyClassHack function
+
+        :param string KB_file: path of the KB instance
+        :param string elementName: name of the element to be added
+    """
+
+    with open(KB_file, "r") as file:
+        data = file.read()
+
+    # Prefix replacement
+    if classID =="Terminator":
+        target = '      <has'+classID+' hasName="'+className+'"/>'
+        replacement = '      <has'+classID+' hasName="'+className+'">\n        <hasStopCriterion/>\n      </has'+classID+'>'
+    elif classID =="RuleSelector" or classID =="RuleEvaluator":
+        target = '      <has' + classID + ' hasName="' + className + '"/>'
+        replacement = '      <has' + classID + ' hasName="' + className + '">\n        <hasDFARule/>\n      </has' + classID + '>'
+
+    with open(KB_file, "w") as file:
+        data_new = data.replace(target, replacement)
+        file.write(data_new)
+
+    file.close()
+
+def EmptySubClassHack(API=None,classID ="Terminator" ,className="Default"):
+    """
+        Custom hack function to generate an empty subclass element
+
+        :param string KB_file: path of the KB instance
+        :param string elementName: name of the element to be added
+
+        :return object model: KB instance model
+    """
+
+    preProcess_emptySubClass(KB_file=API.KB_path,classID =classID ,className=className)
+    API.model = importInstanceModel(API.ECORE_path, API.KB_path, VERBOSE=False)
+    return API.model
+
+
+def postProcess_StopCondition( output_file):
+    """
+        Custom postprocessing function to clean the xmi file defined by an Ecore model
+
+        :param string path: path of the output xmi file
+    """
+    # open file to manually add missing attribute in root element (@ToDo clean this up)
+    with open(output_file, "r") as file:
+        data = file.read()
+
+    # Prefix replacement
+    target = '<includesStopCriterion'
+    replacement = '<hasStopCriterion'
+
+    with open(output_file, "w") as file:
+        data_new = data.replace(target, replacement)
+        file.write(data_new)
+
+    file.close()
 #END DEBUG EXPERIMENT
\ No newline at end of file
