@@ -502,9 +502,56 @@ class AssemblySequence():
             self.AssemblyOptions = AssemblyOptions
 
         else:
+            self.rootAssembly=None
+            self.childActive = False
             self.json2object(JSONDescriptor)
             x = 10
 
+    def setAssemblyOption(self, Option):
+        try:
+            # operations
+            operationList = []
+            for operation in Option['Operations']:
+                op = Operation(Name=operation['Name'], Description="NOT IN MM")
+                op.OperationMetric = [operation['OperationMetric'][0], operation['OperationMetric'][1]]  # TODO: in MM, multiple metrics can be added -> is this necessary?
+                # operators
+                operatorList = []
+                for operator in operation['Operators']:
+                    operator_temp = Operator(Name=operator['Name'],CostPerHour=operator['CostPerHour'],Height=operator['Height'],MaxLiftingWeight=operator['MaxLiftingWeight'],Reach=operator['Reach'])
+                    operatorList.append(operator_temp)
+                op.Operators = operatorList
+                # Tools
+                ToolList = []
+                for tool in operation['Tool']:
+                    requiredOperators = []
+                    for r_operator in operation['Operators']:
+                        requiredOperators.append(r_operator['Name'])  # TODO: only name reference added, ok?
+                    Tool_temp = Tool(Name=tool['Name'],Mass=tool['Mass'],Cost=tool['Cost'],Orientation=tool['Orientation'],Geometry=tool['Geometry'],RequiredOperators=requiredOperators)
+                    #Tool_temp = [tool.hasName, tool.hasMass, tool.hasCost, tool.hasOrientation.hasData,tool.hasGeometry.hasDataInStepFile, requiredOperators]
+                    ToolList.append(Tool_temp)
+                op.Tool = ToolList
+                # Fastners
+                FastnersList = []
+                for fastner in operation['Fastners']:
+                    FastnersList.append(fastner[0])  # TODO: only name reference added, ok?
+                op.Fastners = FastnersList
+                operationList.append(op)
+
+            # -- define the assemblyOption --
+            ASO = AssemblyOption(Name=Option['Name'],Operations=operationList)
+
+            #Linking the recursive operations!
+            subAssemblyOptions = []
+            for recursive_option in Option['RecursiveAssemblyOption']:
+                subAssemblyOptions.append(self.setAssemblyOption(recursive_option))
+
+            ASO.RecursiveAssemblyOption = subAssemblyOptions
+
+            return ASO
+
+        except:
+            return []
+
     def json2object(self, jsonDescriptor):
         """
              Function to generate a AssemblySequence object from a JSON file
@@ -516,7 +563,109 @@ class AssemblySequence():
         # --interpret JSON file--
         with open(jsonDescriptor, "r") as read_file:
             jsonObject = json.load(read_file)
+            self.Name=jsonObject['Name']
+            self.Description = jsonObject['Description']
+            self.AssemblyMetric = [jsonObject['AssemblyMetric'][0],jsonObject['AssemblyMetric'][1]]
 
+            # -- recursive AssemblyOptions --
+            self.AssemblyOptions = self.setAssemblyOption(Option=jsonObject['AssemblyOptions'])
+
+
+
+
+    def object2json(self):
+        """
+               Function to generate a json file from the AssemblySequence model
+        """
+        return json.dumps(self, default=lambda o: o.__dict__, indent=4)
+
+class AssemblyOption():
+    """
+            AssemblyOption: Class representing the AssemblyOption
+
+             :param string Name: Name of the AssemblyOption
+             :param float Description: Cost of the AssemblyOption
+             :param ObjectList Operations: OperationList of the AssemblyOption
+             :param list OperatorMetric: OperatorMetric of the AssemblyOption
+             :param ObjectList Operators: List of required operators of the AssemblyOption
+             :param objectList ToolList: Required tools of the AssemblyOption
+             :param objectList RecursiveAssemblyOption: Nested AssemblyOption
+
+    """
+    def __init__(self, JSONDescriptor=None, Name="", Description="",Operations=[],RecursiveAssemblyOption=[], DEBUG=True):
+        self.DEBUG = DEBUG
+        if JSONDescriptor is None:
+            self.Name = Name
+            self.Description = Description
+            self.Operations = Operations
+            self.RecursiveAssemblyOption = RecursiveAssemblyOption
+
+        else:
+            # TODO: import json descriptor to setup object
+            x = 1
+
+    def object2json(self):
+        """
+               Function to generate a json file from the AssemblySequence model
+        """
+        return json.dumps(self, default=lambda o: o.__dict__, indent=4)
+
+class Operator():
+    """
+            Operator: Class representing the Operator
+
+             :param string Name: Name of the Operator
+             :param float CostPerHour: Cost of the Operator
+             :param float Height: Height of the Operator in cm
+             :param float MaxLiftingWeight: Maximum lifting capabilities of the Operator
+             :param float Reach: Reach capabilities of the Operator
+
+    """
+    def __init__(self, JSONDescriptor=None, Name="", CostPerHour=70.0,Height=180.0,MaxLiftingWeight=50, Reach=155.0, DEBUG=True):
+        self.DEBUG = DEBUG
+        if JSONDescriptor is None:
+            self.Name = Name
+            self.CostPerHour = CostPerHour
+            self.Height = Height
+            self.MaxLiftingWeight = MaxLiftingWeight
+            self.Reach = Reach
+
+        else:
+            # TODO: import json descriptor to setup object
+            x = 1
+
+    def object2json(self):
+        """
+               Function to generate a json file from the AssemblySequence model
+        """
+        return json.dumps(self, default=lambda o: o.__dict__, indent=4)
+
+class Tool():
+    """
+            Tool: Class representing the Tool   tool.hasName, tool.hasMass, tool.hasCost, tool.hasOrientation.hasData,
+                                 tool.hasGeometry.hasDataInStepFile, requiredOperators
+
+             :param string Name: Name of the Tool
+             :param float Mass: Mass of the Tool
+             :param float Cost: Cost of the Tool
+             :param string Orientation: Orientation data of the Tool
+             :param string hasGeometry: hasGeometry info of the Tool
+             :param List RequiredOperators: List of required tool operators (referenced by name)
+
+    """
+    def __init__(self, JSONDescriptor=None, Name="", Mass=70.0,Cost=500.0,Orientation="",Geometry="", RequiredOperators=[], DEBUG=True):
+        self.DEBUG = DEBUG
+        if JSONDescriptor is None:
+            self.Name = Name
+            self.Mass = Mass
+            self.Cost = Cost
+            self.Orientation = Orientation
+            self.Geometry = Geometry
+            self.RequiredOperators = RequiredOperators
+
+        else:
+            # TODO: import json descriptor to setup object
+            x = 1
 
     def object2json(self):
         """
@@ -571,6 +720,9 @@ class Operation():       #
         """
         return json.dumps(self, default=lambda o: o.__dict__, indent=4)
 
+
+#---------------------Ontology & contracts interface objects--------------------------
+
 class Contract():       #
     """
         Contract: Class representing the Contract
