@@ -38,7 +38,7 @@ from .PerformanceFunctions import *
 from .ContractFunctions import *
 
 #interface objects
-from .InterfaceObjects import ASG,StopCondition,DFARule,Product,Parameter,ProductPart,AssemblySequence,Operation
+from .InterfaceObjects import ASG,StopCondition,DFARule,Product,Parameter,ProductPart,AssemblySequence,Operation,Operator,Tool,AssemblyOption
 
 
 
@@ -413,7 +413,7 @@ class KB_Interface():
         else:
             return -1  # no ProductPart instance
 
-    def setProductPart(self, interfaceObject):
+    def setProductPart(self, interfaceObject,triggerFault = False):
         """
              Function to add a ProductPart model to an blank KB .
              The ProductPart interface object is used to interface with the function.
@@ -437,17 +437,17 @@ class KB_Interface():
         # productPart.Quantity = interfaceObject.Quantity                 # TODO:Quantity not in MM v6
 
 
-        #TODO: material and model not correctly added!
-
         #add model to ProductPart
         model = self.create("Model")
+        model.hasModelType = ""
         part.hasModel = model
         # update the parameters
-        parameter = self.create("Parameter")
+
         for par_new in interfaceObject.parameterList:
+            parameter = self.create("Parameter")
             parameter.hasKey = par_new.Key
             parameter.hasValue = par_new.Value
-            part.hasModel.hasModelParameter.items.append(parameter)
+            part.hasModel.hasModelParameter.append(parameter)
 
         # link the product parts - Recursive!
         #for par in productPart.hasCompositionEdge.items:
@@ -471,6 +471,11 @@ class KB_Interface():
         part.hasMaterial.hasStiffness = stiffness
         part.hasMaterial.hasStiffness.hasScalarMatrix6x6 = interfaceObject.material[5]
 
+        #--DEBUG PyECORE--
+        if triggerFault:
+            self.model.includesProduct[0].hasProductPart.append(part)
+        #----------------
+
         self.updateKBv6()
 
         return 1
@@ -563,7 +568,7 @@ class KB_Interface():
                         #assemblyOption - recursive
                         assemblyOptions = self.getAssemblyOption(s_temp.hasAssemblyOption)
 
-                        sequence = AssemblySequence(Name=assemblyOptions[0], Description="NOT IN MM",AssemblyMetric=assemblyMetric,AssemblyOptions=assemblyOptions)
+                        sequence = AssemblySequence(Name=assemblyOptions.Name, Description="NOT IN MM",AssemblyMetric=assemblyMetric,AssemblyOptions=assemblyOptions)
 
                         return sequence
 
@@ -590,7 +595,7 @@ class KB_Interface():
         return -1
 
 
-    def setAssemblySequence(self,AssemblySystemName,IntefaceObject):
+    def setAssemblySequence(self,AssemblySystemName,InterfaceObject):
         """
              Function to add a AssemblySequence model to an existing AssemblySystem using the name of the assemblySystem as identifier.
              The AssemblySequence interface object is used to interface with the function.
@@ -601,16 +606,65 @@ class KB_Interface():
 
              :return int Error: -1 = error, 1= function performed correcly
         """
-        x = 10
+        self.initialize(self.ECORE_path)
+        AssemblySequence = self.create("AssemblySequence")
+
+        #----add AssemblyMetric----- TODO: single metric for now, ok?
+        asm = self.create("AssemblyMetric")
+        asm.hasName=InterfaceObject.AssemblyMetric[0]
+        asm.hasValue = InterfaceObject.AssemblyMetric[1]
+        AssemblySequence.hasAssemblyMetric.append(asm)
+
+        # -- recursive AssemblyOptions --
+        AssemblySequence.hasAssemblyOption = self.setAssemblyOption_NEW(Option=InterfaceObject.AssemblyOptions)
+
+        #-----------------------------------------------------------------------------------------------------
+        #   DEBUG PURPOSE: Add product and assemblySystem if not exist
+        #-----------------------------------------------------------------------------------------------------
+        if len(self.model.includesAssemblySystem.items)==0:
+
+            # ---------------------- PRODUCT DUMMY ---------------------------
+            product = self.create("Product")
+            product.hasProductName = "DUMMY-PRODUCT-DEBUG"
+            product.hasSTEPfile = 'c:/NEW/PATH/TO/THE/STEPFILE'
+
+
+            # ---------------------- ASSEMBLY SYSTEM DUMMY ---------------------------
+            asys = self.create("AssemblySystem")
+
+            # -- minimal dependecies --
+            asys.hasName = AssemblySystemName
+            asys.hasGUID = ""
+            GravityDirection = self.create("Direction")
+            asys.hasGravityDirection = GravityDirection
+
+            GroundPosition = self.create("Position")
+            asys.hasGroundPosition = GroundPosition
+
+        else:
+            # ---------------------- PRODUCT REAL ---------------------------
+            product = self.model.includesProduct.items[0]
+            # ---------------------- ASSEMBLY SYSTEM REAL ---------------------------
+
+            # Search for corresponding assemblySystem and add - ASSUMPTION: everything else is set for the assemblySystem
+            for a in self.model.includesAssemblySystem.items:
+                if a.hasName == AssemblySystemName:
+                    asys = a
+
+        asys.hasAssemblySequence = AssemblySequence
+        product.hasAssemblySystem.append(asys)
+
+        self.updateKBv6()
+
         return -1
 
 
-    def getAssemblyOption(self,AssemblyOption):
+    def getAssemblyOption(self,Option):
         """
          Helper function to fetch the Assembly options for each Assembly step .
          The function returns a recursive AssemblyOption List
 
-        :param object AssemblyOption: Assembly option ecore object
+        :param object Option: Assembly option ecore object
 
          :return List AssemblyOptionList: list of assembly options RECURSIVE (-1 = error)
         """
@@ -619,15 +673,13 @@ class KB_Interface():
         try:
             # operations
             operationList = []
-            for operation in AssemblyOption.hasOperation.items:
+            for operation in Option.hasOperation.items:
                 op = Operation(Name=operation.hasName, Description="NOT IN MM")
-                op.OperationMetric = [operation.hasOperationMetric.items[0].hasName, operation.hasOperationMetric.items[
-                    0].hasValue]  # TODO: in MM, multiple metrics can be added -> is this necessary?
+                op.OperationMetric = [operation.hasOperationMetric.items[0].hasName, operation.hasOperationMetric.items[0].hasValue]  # TODO: in MM, multiple metrics can be added -> is this necessary?
                 # operators
                 operatorList = []
                 for operator in operation.hasOperator.items:
-                    operator_temp = [operator.hasName, operator.hasCostPerHour, operator.hasHeight,
-                                     operator.hasMaxLiftingWeight, operator.hasReach]
+                    operator_temp = Operator(Name=operator.hasName,CostPerHour=operator.hasCostPerHour,Height=operator.hasHeight,MaxLiftingWeight=operator.hasMaxLiftingWeight,Reach=operator.hasReach)
                     operatorList.append(operator_temp)
                 op.Operators = operatorList
                 # Tools
@@ -636,8 +688,8 @@ class KB_Interface():
                     requiredOperators = []
                     for r_operator in operation.hasOperator.items:
                         requiredOperators.append(r_operator.hasName)  # TODO: only name reference added, ok?
-                    Tool_temp = [tool.hasName, tool.hasMass, tool.hasCost, tool.hasOrientation.hasData,
-                                 tool.hasGeometry.hasDataInStepFile, requiredOperators]
+                    Tool_temp = Tool(Name=tool.hasName,Mass=tool.hasMass,Cost=tool.hasCost,Orientation=tool.hasOrientation.hasData,Geometry=tool.hasGeometry.hasDataInStepFile,RequiredOperators=requiredOperators)
+                    #Tool_temp = [tool.hasName, tool.hasMass, tool.hasCost, tool.hasOrientation.hasData,tool.hasGeometry.hasDataInStepFile, requiredOperators]
                     ToolList.append(Tool_temp)
                 op.Tool = ToolList
                 # Fastners
@@ -647,23 +699,135 @@ class KB_Interface():
                 op.Fastners = FastnersList
                 operationList.append(op)
 
+            # -- define the assemblyOption --
+            ASO = AssemblyOption(Name=Option.hasName,Operations=operationList)
+
+            #Linking the recursive operations!
+            subAssemblyOptions = []
+            for recursive_option in Option.hasRecursiveAssemblyOption:
+                subAssemblyOptions.append(self.getAssemblyOption(recursive_option))
+
+            ASO.RecursiveAssemblyOption = subAssemblyOptions
+
+            return ASO
+
+        except:
+            return []
+
+
+    def setAssemblyOption_NEW(self, Option):
+        try:
+
+            # -- define the assemblyOption --
+            ASO = self.create("SubAssembly")
+            ASO.hasName = Option.Name
+
+            # operations
+            for operation in Option.Operations:
+                op = self.create("Operation")
+
+                # ----add OperationMetric-----
+                om = self.create("OperationMetric")
+                om.hasName = operation.OperationMetric[0]
+                om.hasValue = operation.OperationMetric[1]
+                op.hasOperationMetric.append(om)
+
+                # ----add Fastners----
+                # TODO: productParts by name?
+
+                # ----add Operators-----
+                # TODO:implement after hacks are removed!
+
+                # ----add Tools-----
+                for tool in operation.Tool:
+                    t = self.create("Tool")
+                    t.hasName = tool.Name
+                    t.hasCost = tool.Cost
+                    t.hasMass = tool.Mass
+
+                    # --# ----Geometry-----
+                    g = self.create("Geometry")
+                    g.hasDataInStepFile = tool.Geometry
+                    t.hasGeometry = g
+                    # --# ----Orientation-----
+                    o = self.create("Orientation")
+                    o.hasData = tool.Orientation
+                    t.hasOrientation = o
+
+                    op.hasTool.append(t)
+
+                    # -- Add operation to subassembly --
+                    ASO.hasOperation.append(op)
+
+
 
 
             #Linking the recursive operations!
             subAssemblyOptions = []
-            for recursive_option in AssemblyOption.hasRecursiveAssemblyOption:
-                subAssemblyOptions.append(self.getAssemblyOption(recursive_option))
+            for recursive_option in Option.RecursiveAssemblyOption:
+                tmp = self.setAssemblyOption_NEW(recursive_option)
+                ASO.hasRecursiveAssemblyOption.append(tmp)
 
-            AssemblyOptionList = [AssemblyOption.hasName, operationList,subAssemblyOptions]
+            #ASO.RecursiveAssemblyOption = subAssemblyOptions
 
-            return AssemblyOptionList
+            return ASO
 
         except:
+            return []
+
+
+    def setAssemblyOption(self, AssemblyOption):
+        #non-recursive for now, TODO:EXTEND
+        try:
+            aso = self.create("SubAssembly")
+            aso.hasName = AssemblyOption.Name
+            #aso.hasDescription = AssemblyOption.Description
+
+            # --- add assembly operation ---
+            op = self.create("Operation")
+            # ----add Fastners----
+                #TODO: productParts by name?
+            # ----add OperationMetric-----
+            om = self.create("OperationMetric")
+            om.hasName = AssemblyOption.OperatorMetric[0]
+            om.hasValue = AssemblyOption.OperatorMetric[1]
+            op.hasOperationMetric.append(om)
+
+            # ----add Operators-----
+                #TODO:implement after hacks are removed!
+            # ----add Tools-----
+            for tool in AssemblyOption.ToolList:
+                t = self.create("Tool")
+                t.hasName = tool.Name
+                t.hasCost = tool.Cost
+                t.hasMass = tool.Mass
+
+                #--# ----Geometry-----
+                g = self.create("Geometry")
+                g.hasDataInStepFile = tool.Geometry
+                t.hasGeometry=g
+                # --# ----Orientation-----
+                o = self.create("Orientation")
+                o.hasData = tool.Orientation
+                t.hasOrientation=o
+
+                op.hasTool.append(t)
+
+            aso.hasOperation.append(op)
+
+            # --- add the recursive operations ---
+            for recursive_option in AssemblyOption.RecursiveAssemblyOption:
+                temp = self.setAssemblyOption(AssemblyOption=recursive_option)
+                AssemblyOption.hasRecursiveAssemblyOption.append(temp)
+
+            return aso
+
+        except:
+
+            if self.DEBUG:print("ERROR - No option added")
             return -1
 
 
-
-
 # -----------------------------------------------------------------------------------------------------------
 #
 #   Contract-Ontology extension
